﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Inventor;

namespace SynchFeatureTestApp
{
    public partial class Form1 : Form
    {
        private Inventor.Application inventor;
        private bool isSessionStartedByThisApp;
        private Document activeDocument;

        public Form1()
        {
            InitializeComponent();
            //this.textBoxDocumentPath.Text = @"C:\Temp\Bracket.ipt";
        }

        private Inventor.Application Connect()
        {
            string InventorProgID = "Inventor.Application";
            Inventor.Application inventorApp = null;

            try
            {
                // Connect to a running instance of Solid Edge.
                inventorApp = (Inventor.Application)Marshal.GetActiveObject(InventorProgID);
            }
            catch
            {
            }

            if (null == inventorApp)
            {
                // Get the type from the Solid Edge ProgID.
                Type type = Type.GetTypeFromProgID(InventorProgID);

                // Start new instance Solid Edge.
                inventorApp = (Inventor.Application)Activator.CreateInstance(type);
                if (null != inventorApp)
                {
                    this.isSessionStartedByThisApp = true;
                }
            }

            return inventorApp;
        }

        private void Disconnect()
        {
            if (this.isSessionStartedByThisApp && null != this.inventor)
            {
                this.inventor.Quit();
                Marshal.ReleaseComObject(this.inventor);
            }

            this.inventor = null;
        }

        private void buttonConnect_Click(object sender, EventArgs e)
        {
            this.inventor = Connect();

            if (this.inventor != null)
                MessageBox.Show("Successfully connected to Inventor session.");
            else
                MessageBox.Show("Failed to connect Inventor session.");
        }

        private void buttonQuit_Click(object sender, EventArgs e)
        {
            Disconnect();
            MessageBox.Show("Inventor session disconnected.");
        }

        private bool OpenDocument(string fullFileName, ref Document document)
        {
            Debug.Assert(!String.IsNullOrEmpty(fullFileName), "fullFileName is null or empty.");
            bool successState = false;

            try
            {
                document = inventor.Documents.Open(fullFileName, false);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
                return successState;
            }

            if ((null != document))
                successState = true;

            return successState;
        }

        private void CloseDocument(ref Document document)
        {
            if (document == null)
                return;

            //if (document.Views.Count > 0)
            //    document = null;
            //else
                document.Close(true);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            this.inventor = Connect();
            bool success = OpenDocument(this.textBoxDocumentPath.Text, ref this.activeDocument);

            if (success)
                MessageBox.Show("Document Open: Successful.");
            else
                MessageBox.Show("Document Open: Failed.");
        }

        private void button2_Click(object sender, EventArgs e)
        {
            CloseDocument(ref this.activeDocument);
            Disconnect();
        }

        private void buttonUnfold_Click(object sender, EventArgs e)
        {
            PartDocument partDocument = this.activeDocument as PartDocument;
            Debug.Assert(null != partDocument, "partDocument is null.");
            SheetMetalComponentDefinition sheetMetalDefinition = partDocument.ComponentDefinition as SheetMetalComponentDefinition;
            Debug.Assert(null != sheetMetalDefinition, "sheetMetalDefinition is null.");

            ErrorManager errorManager = this.inventor.ErrorManager;
            try
            {
                if (!sheetMetalDefinition.HasFlatPattern)
                {
                    sheetMetalDefinition.Unfold();
                    MessageBox.Show("Unfold Part: Suceessful.");
                }
            }
            catch (Exception exp)
            {
                string messsage = "Exception Message: " + exp.Message + System.Environment.NewLine +
                                  "ErrorManager Message: " + errorManager.LastMessage;
                MessageBox.Show(exp.Message);
            }
        }
    }
}
